{{

Spin code by Clock Loop.

The code communicates with the wx module and a propeller that is connected to a Memsic2125
The index.html accelerometer output displays the updated values every 3 seconds.
As usual upload the index html to your wx module: Visit http://192.168.4.1/files/index.html
}}

CON 

  _CLKMODE  = XTAL1 + PLL16X                              ' The system clock spec
  _XINFREQ  = 5_000_000                                  ' Crystal

  Rx        = 31    'serial in from wx device
  Tx        = 30    'serial out to wx device
                  
  AccXin    = 3     'X-axis PWM input.   Each axis has a 100 Hz PWM duty cycle output in which acceleration is proportional to the ratio tHx/Tx.
  AccYin    = 27    'Y-axis PWM input.   Each axis has a 100 Hz PWM duty cycle output in which acceleration is proportional to the ratio tHx/Tx.          
  
  'AccTemp   = 20   'Not used in this demo
                    'Temperature (Analog Voltage) This pin is the buffered output of the temperature sensor.  
                    'The analog voltage at TOUT is an indication of the die temperature.  
                    'This voltage is useful as a differential measurement of temperature from ambient
                    ' and not as an absolute measurement of temperature. 
  
OBJ

    ser:    "Parallax Serial Terminal.spin"
    MM2125: "Memsic2125"

VAR 

    byte  Handle, ReplyStatus, XTiltListener, YTiltListener
    long  Comm[100] 

Pub Main      'Cog #1
 
  cognew(Comms, @Comm)   'Cog #2 and #3
  
  waitcnt(80_000_000 + cnt) 'Wait for 1 s
                               
  Accelerometer   'Start accelerometer loop. Cog #1 and #4
                                                         
PUB Accelerometer | x, y   'Cog #1 and #4
  
  MM2125.start(AccXin, AccYin)        ' Initialize Memsic 2125      Cog #4  
  waitcnt(80_000_000 + cnt) 'Wait for 1s. 
                             
  repeat    'Cog #1 loop
    x := MM2125.Mx                ' Read X axis
    y := MM2125.My                ' Read Y axis
    
    long[@TiltX][0] := x / 100
    long[@TiltY][0] := y / 100
   
    WaitCnt(ClkFreq / 2 + Cnt)    'wait                                                  

Pub Comms     'Cog #2 and #3

   
  dira[TX]~~          ' Set direction to output
  outa[TX]:= 0        ' Set TX low.                         
  waitcnt(1_000_000 + cnt)   'wait 1/10 second for command mode to start in the module.
  dira[TX]~           ' Set direction to input  
  
  ser.StartRxTx(RX, TX, 0, 115200)   'Cog #3  Parallax Serial Terminal does NOT work at receiving data 230400 for some reason.
  waitcnt(1_000_000 + cnt)   'wait 1/10 second for serial to start.
  
                         '
  ser.rxflush                     
  ser.char(254)  'Begin marker
  ser.str(string("LISTEN:HTTP,/TiltX"))   'Tell the module to listen to a specific path on http port 80.
  ser.char(13)    'End marker     
  waitcnt(1_000_000 + cnt) 'Wait for 1/10 s  Let buffer fill.
  Reply[0] := ser.rxcheck   ' should be 254
  Reply[1] := ser.rxcheck   ' should be =
  Reply[2] := ser.rxcheck   ' should be S
  Reply[3] := ser.rxcheck   ' should be ,
  Reply[4] := ser.rxcheck   ' should be 1   Use this returned listener id with PATH or CLOSE commands. 
                            ' There are a maximum of four listeners available. 
                            ' Issuing additional LISTEN commands when all four listeners are already established,
                            '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
  Reply[5] := ser.rxcheck   ' should be 13
  
  If Reply[0] == 254 And Reply[1] == "=" And Reply[2] == "S" And Reply[3] == "," And Reply[4] == "1" And Reply[5] == 13 
    XTiltListener := Reply[4]  
  Else
    ser.str(string(27))      'Set ansi command mode in putty.                 
    ser.str(string("[2;3f")) 'Set Position in putty.      
    ser.str(string("ERROR, wrong X tilt listener returned.")) 'Tell putty error.                       
    ser.str(string(27))      'Set ansi command mode in putty.      
    ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0  
    Repeat
      waitcnt(80_000_000 + cnt) 'Wait for 1s forever.
  
  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data
  
  ser.rxflush                     
  ser.char(254)  'Begin marker
  ser.str(string("LISTEN:HTTP,/TiltY"))   'Tell the module to listen to a specific path on http port 80.
  ser.char(13)    'End marker     
  waitcnt(1_000_000 + cnt) 'Wait for 1/10 s  Let buffer fill.
  Reply[0] := ser.rxcheck   ' should be 254
  Reply[1] := ser.rxcheck   ' should be =
  Reply[2] := ser.rxcheck   ' should be S
  Reply[3] := ser.rxcheck   ' should be ,
  Reply[4] := ser.rxcheck   ' should be 1   Use this returned listener id with PATH or CLOSE commands. 
                            ' There are a maximum of four listeners available. 
                            ' Issuing additional LISTEN commands when all four listeners are already established,
                            '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
  Reply[5] := ser.rxcheck   ' should be 13
  
  If Reply[0] == 254 And Reply[1] == "=" And Reply[2] == "S" And Reply[3] == "," And Reply[4] == "2" And Reply[5] == 13 
    YTiltListener := Reply[4]  
  Else
    ser.str(string(27))      'Set ansi command mode in putty.                 
    ser.str(string("[2;3f")) 'Set Position in putty.      
    ser.str(string("ERROR, wrong Y tilt listener returned.")) 'Tell putty error.                       
    ser.str(string(27))      'Set ansi command mode in putty.      
    ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0  
    Repeat
      waitcnt(80_000_000 + cnt) 'Wait for 1s forever.
  
  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data
      
  ser.str(string(27))      'Set ansi command mode in putty.                 
  ser.str(string("[2;3f")) 'Set Position in putty.      
  ser.str(string("Prop Communication Started")) 'Tell putty command mode is on.                       
  ser.str(string(27))      'Set ansi command mode in putty.      
  ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0        
  Repeat 10  ' wait 10 seconds to help determine if prop reset.
    waitcnt(80_000_000 + cnt)   'wait 1 second 
        
  Repeat                'The main repeat loop.  Cog#2
    ser.str(string(27))     'Set ansi command mode in putty.
    ser.str(string("[2J"))  'Clear putty telnet screen. 
    
    ser.str(string(27))      'Set ansi command mode in putty.                 
    ser.str(string("[2;3f")) 'Set Position in putty.      
    ser.str(string("Command Mode is On. ")) 'Tell putty command mode is on.                       
    ser.str(string(27))      'Set ansi command mode in putty.      
    ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0

    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[4;3f")) 'Set Position in putty
                                                           
    If ReplyStatus == 1     
      ser.str(string("Good reply."))   'If last reply was good, put it back up on putty which was cleared.
      ReplyStatus := 0                                 '                
    Else
      ser.str(string("  No reply."))    'If last reply was none, put it back up on putty which was cleared.   

    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[5;3f")) 'Set Position in putty 
    ser.str(string("Tilt X that the Prop sent: ")) '
    ser.dec(long[@TiltX][0])
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[6;3f")) 'Set Position in putty 
    ser.str(string("Tilt Y that the Prop sent: ")) '
    ser.dec(long[@TiltY][0])
                                              
    ser.str(string(27))      'Set ansi command mode in putty.                                               
    ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0                         
    ser.char(13)    'End marker
  
    Reply[0] := 0  ' clear data
    Reply[1] := 0  ' clear data
    Reply[2] := 0  ' clear data
    Reply[3] := 0  ' clear data
    Reply[4] := 0  ' clear data
    Reply[5] := 0  ' clear data
    Reply[6] := 0  ' clear data
    Reply[7] := 0  ' clear data
      
    Repeat 400
    
        Reply[0] := 0  ' clear data
        Reply[1] := 0  ' clear data
        Reply[2] := 0  ' clear data
        Reply[3] := 0  ' clear data
        Reply[4] := 0  ' clear data
        Reply[5] := 0  ' clear data
        Reply[6] := 0  ' clear data
        Reply[7] := 0  ' clear data
    
        ser.rxflush
        ser.char(254)  'Begin marker
        ser.str(string("POLL:"))   'Tell the module to listen to a ANY protocol/path.
        ser.char(13)    'End marker
        waitcnt(2_000_000 + cnt) 'Wait for 1/10 s  Let buffer fill.
        Reply[0] := ser.rxcheck   ' should be 254
        Reply[1] := ser.rxcheck   ' should be =
        Reply[2] := ser.rxcheck   ' should be P
        Reply[3] := ser.rxcheck   ' should be ,
        Reply[4] := ser.rxcheck   ' should be 5 (this could change) The connection identifier to use for this request
        Reply[5] := ser.rxcheck   ' should be ,
        Reply[6] := ser.rxcheck   ' should be 1 or a 2. Use this returned listener id with PATH or CLOSE commands. 
                                  ' There are a maximum of four listeners available. 
                                  ' Issuing additional LISTEN commands when all four listeners are already established,
                                  '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
        Reply[7] := ser.rxcheck   ' should be 13   
                                           
        
        If Reply[0] == 254 And Reply[1] == "=" And Reply[2] == "G" And Reply[6] == XTiltListener And Reply[7] == 13
            ReplyStatus := 1     'Set the reply to good so it can be shown in putty.
          
            Handle := Reply[4]  'Store connection identifier.
          
            Reply[0] := 0  ' clear data
            Reply[1] := 0  ' clear data
            Reply[2] := 0  ' clear data
            Reply[3] := 0  ' clear data
            Reply[4] := 0  ' clear data
            Reply[5] := 0  ' clear data
            Reply[6] := 0  ' clear data
            Reply[7] := 0  ' clear data                  
            
            ser.rxflush             
            ser.char(254)               'Begin marker
            ser.str(string("REPLY:"))   'Retrieve HTTP GET/POSTâ€™s name argument (in query or body) on connection handle.
            ser.char(Handle)            'An active connection handle; returned by POLL
            ser.str(string(","))        'Required for a reply
            ser.str(string("200"))      'The desired HTTP response code for the reply.                         
            ser.str(string(","))        'Required for a reply   
            ser.str(string("4"))        'Tell the module how many bytes? (or is this digits?) the reply is.
            ser.char(13)                'End marker                                        
            ser.dec(long[@TiltX][0])              'Send 1 byte (which is up to 4 characters (in decimal) (xxxx) of data to the module which then sends it to the webpage.                                                                '                                              
        
        If Reply[0] == 254 And Reply[1] == "=" And Reply[2] == "G" And Reply[6] == YTiltListener And Reply[7] == 13
            ReplyStatus := 1     'Set the reply to good so it can be shown in putty.
          
            Handle := Reply[4]  'Store connection identifier.
          
            Reply[0] := 0  ' clear data
            Reply[1] := 0  ' clear data
            Reply[2] := 0  ' clear data
            Reply[3] := 0  ' clear data
            Reply[4] := 0  ' clear data
            Reply[5] := 0  ' clear data
            Reply[6] := 0  ' clear data
            Reply[7] := 0  ' clear data                  
            
            ser.rxflush             
            ser.char(254)               'Begin marker
            ser.str(string("REPLY:"))   'Retrieve HTTP GET/POSTâ€™s name argument (in query or body) on connection handle.
            ser.char(Handle)            'An active connection handle; returned by POLL
            ser.str(string(","))        'Required for a reply
            ser.str(string("200"))      'The desired HTTP response code for the reply.                         
            ser.str(string(","))        'Required for a reply   
            ser.str(string("4"))        'Tell the module how many bytes? (or is this digits?) the reply is.
            ser.char(13)                'End marker                                        
            ser.dec(long[@TiltY][0])    'Send 1 byte (which is up to 4 characters (in decimal) (xxxx) of data to the module which then sends it to the webpage.                             
   
   

DAT
  TiltX             long  0
  TiltY             long  0
  Entry             byte  0
  'Byte[#]               '0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 18 20
  Reply             byte  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
{{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                  TERMS OF USE: MIT License
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
// Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////      
}}   